<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use App\User;

/**
 * Authentication token which authorizes user to perform API calls
 */
class AuthToken extends Model
{
    /**
     * Constructor for AuthToken
     *
     * @param string|null  $token
     * @return Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function __construct($token = null)
    {
        $this->auth_token = $token;
        $this->expires_at = now()->addDays(15);
    }

    /**
     * Retreives the token with caching
     *
     * @param string $token
     * @return App\AuthToken
     */
    public function GetWithToken($token)
    {
        return $this->GetWithIdentifier('auth_token', $token);
    }

    /**
     * Retreives the token with caching
     *
     * @param string  $identifier
     * @param string $token
     * @return App\AuthToken
     */
    public function GetWithIdentifier($identifier, $token)
    {
        return Cache::remember(static::class, 'GetWithIdentifier', $token, 600, function () use ($identifier, $token) {
          return $this->where($identifier, $token)->first();
        });
    }

    /**
     * Removes the auth token from cache and deletes it
     *
     */
    public function DeleteAndForget()
    {
        Cache::forget(static::class, 'GetWithIdentifier', $this->auth_token);
        $this->delete();
    }

    /**
     * Has one relation to \App\User model
     *
     * @return Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function User()
    {
        return $this->belongsTo('App\User', 'user_id', 'Id');
    }
}
