<?php

namespace App\Exceptions;

use Exception;
use Illuminate\Foundation\Exceptions\Handler as ExceptionHandler;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Illuminate\Auth\AuthenticationException;
use App\Exceptions\AppException;

class Handler extends ExceptionHandler
{
    /**
     * A list of the exception types that are not reported.
     *
     * @var array
     */
    protected $dontReport = [
        //
    ];

    /**
     * A list of the inputs that are never flashed for validation exceptions.
     *
     * @var array
     */
    protected $dontFlash = [
        'password',
        'password_confirmation',
    ];

    /**
     * Report or log an exception.
     *
     * @param  \Exception  $exception
     * @return void
     */
    public function report(Exception $exception)
    {
        parent::report($exception);
    }

    /**
     * Render an exception into an HTTP response.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Exception  $exception
     * @return \Illuminate\Http\Response
     */
    public function render($request, Exception $exception)
    {
        if ($request->ajax()) {
            if($exception instanceof AppException) {
                return \App\Http\Response::Error($exception->getMessage(), $exception->getStatusCode());
            }

            if($exception instanceof AuthenticationException) {
                return \App\Http\Response::Error("Unauthenticated", 401);
            }

            if($exception instanceof NotFoundHttpException) {
                return \App\Http\Response::Error("API endpoint not found", 404);
            }

            if($exception instanceof HttpException) {
                return \App\Http\Response::Error("Server error", $exception->getStatusCode());
            }

            error_log($exception);

            return \App\Http\Response::Error(
                env('APP_ENV', 'production') === 'production' ? 'Server Error' : $exception->getMessage(),
                500);
        }

        return parent::render($request, $exception);
    }
}
