<?php

namespace App\Helpers;

/**
 *  Helper to set .env file variables persistently
 */

class Environment
{
  public const allowedKeys = [
    'CHECK_SCHEMA_VERSION', 'USE_CACHE',
    'USE_UPLOADS', 'UPLOADS_FETCH_REQUIRES_AUTH', 'UPLOADS_FILE_TYPES', 'UPLOADS_MAX_SIZE', 'UPLOADS_PATH', 'UPLOADS_FULL_URL',
    'USE_REGISTRATION', 'REGISTRATION_DEFAULT_USER_ROLE', 'REGISTRATION_DEFAULT_USER_STATUS',
    'USE_LDAP', 'LDAP_DEFAULT_USER_ROLE', 'LDAP_HOSTS', 'LDAP_PORT', 'LDAP_TIMEOUT', 'LDAP_BASE_DN'
  ];

  public static function getDirect($key, $defaultValue = null) {
    if (!in_array($key, static::allowedKeys)) {
      throw new \App\Exceptions\AppException(422, 'Key '.$key.' is not allowed to be queried');
    }

    $envFilePath = '../.env';
    if (!file_exists($envFilePath)) {
      $envFilePath = '.env';
    }
    error_log($envFilePath);

    $raw = file_get_contents($envFilePath);

    $captures = [];
    $key = preg_quote($key);
    if (preg_match("/^{$key}=(.*)$/m", $raw, $captures)) {
      return $captures[1];
    }

    return $defaultValue;
  }

  public static function getDirectArray($array) {
    foreach ($array as $key) {
      if (!in_array($key, static::allowedKeys)) {
        throw new \App\Exceptions\AppException(422, 'Key '.$key.' is not allowed to be queried');
      }
    }

    $envFilePath = '../.env';
    if (!file_exists($envFilePath)) {
      $envFilePath = '.env';
    }

    $raw = file_get_contents($envFilePath);

    $dictionary = [];
    foreach ($array as $key) {
      $captures = [];
      $key = preg_quote($key);
      if (preg_match("/^{$key}=(.*)$/m", $raw, $captures)) {
        $dictionary[$key] = $captures[1];
      } else {
        $dictionary[$key] = null;
      }
    };

    return $dictionary;
  }

  public static function get($key, $defaultValue = null) {
    if (!in_array($key, static::allowedKeys)) {
      throw new \App\Exceptions\AppException(422, 'Key '.$key.' is not allowed to be queried');
    }

    return env($key, $defaultValue);
  }

  public static function getArray($array) {
    foreach ($array as $key) {
      if (!in_array($key, static::allowedKeys)) {
        throw new \App\Exceptions\AppException(422, 'Key '.$key.' is not allowed to be queried');
      }
    }

    $dictionary = [];
    foreach ($array as $key) {
      $dictionary[$key] = env($key);
    };

    return $dictionary;
  }

  public static function set($key, $value)
  {
    if (!in_array($key, static::allowedKeys)) {
      throw new \App\Exceptions\AppException(422, 'Key '.$key.' is not allowed to be set');
    }

    if (is_string($value) && !preg_match("/^[a-zA-Z0-9\.\,\=\_\-]+$/", $value)) {
      throw new \App\Exceptions\AppException(422, 'Value '.$value.' does not pass validation');
    }

    $path = app()->environmentFilePath();
    if (is_bool($value) === true)
      $value = ($value) ? 'true' : 'false';

    file_put_contents($path, preg_replace(
        "/^{$key}=.*$/m",
        "{$key}={$value}",
        file_get_contents($path)
    ));
  }

  public static function setArray($array)
  {
    foreach ($array as $key => $value) {
      if (!in_array($key, static::allowedKeys)) {
        throw new \App\Exceptions\AppException(422, 'Key '.$key.' is not allowed to be set');
      }

      if (is_string($value) && !preg_match("/^[a-zA-Z0-9\.\,\=\_\-]+$/", $value)) {
        throw new \App\Exceptions\AppException(422, 'Value '.$value.' does not pass validation');
      }
    }

    $path = app()->environmentFilePath();
    $contents = file_get_contents($path);

    foreach ($array as $key => $value) {
    if (is_bool($value) === true)
      $value = ($value) ? 'true' : 'false';

      $contents = preg_replace(
          "/^{$key}=.*$/m",
          "{$key}={$value}",
          $contents);
    }

    file_put_contents($path, $contents);
  }
}
