<?php

/**
* Controller for File model. Generated automatically, do not edit
* Generator by Evgeny Lavrik <e.lavrik@gsi.de>
* 4.6.2019
**/

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Str;

use App\Http\Controllers\BaseController;
use App\Http\Controllers\BaseControllerVersioned;

class FileController extends BaseController
{
  protected $Model = \App\File::class;

  public function ServeFile($path, Request $request) {
    $basePath = storage_path('app');
    $prefix = env('UPLOADS_PATH', 'up');
    $fullPath = "{$basePath}/{$prefix}/{$path}";
    if (!file_exists($fullPath)) {
      return \App\Http\Response::Error("Not found", 404);
    }

    return file_get_contents($fullPath);
  }

  /**
   * Uploads a file to the server
   *
   * @param  Request  $request
   * @return \Response
   */
  public function UploadFile(Request $request) {
    if (!env('USE_UPLOADS', false)) {
      throw new \App\Exceptions\AppException(422, 'File uploads were disabled by administrator');
    }

    $maxSize = env('UPLOADS_MAX_SIZE','5120'); // 5MB

    \App\Http\Request::ValidateInput($request, [
      'file' => 'required|file|max:'.$maxSize,
      'description' => 'nullable',
      'class' => 'nullable',
      'full_url' => 'nullable',
    ]);

    $types = env('UPLOADS_FILE_TYPES', 'jpeg,jpg,png,root,pdf');
    $ext = request()->file->getClientOriginalExtension();
    $cksum = hash_file('sha256', request()->file->path());

    if (!in_array(strtolower($ext), explode(',', $types))) {
      throw new \App\Exceptions\AppException(422, 'Unsupported file type');
    }

    $basePath = storage_path('app');
    $pathElements = [];
    $pathElements[]= $this->sanitize(env('UPLOADS_PATH', 'up'));
    $classPath = $request->request->get('class');
    if ($classPath) {
      $pathElements[]= $this->sanitize($classPath);
    }
    $destinationDir = join('/', array_merge([$basePath], $pathElements));

    if (!is_dir($destinationDir)) {
      mkdir($destinationDir, 0755, true);
    }

    $fileName = null;
    do {
       $fileName = pathinfo(request()->file->getClientOriginalName(), PATHINFO_FILENAME).'.'.Str::random(3).'.'.$ext;
       $fileName = $this->sanitize($fileName);
    } while(file_exists("{$destinationDir}/{$fileName}"));
    $request->file->move($destinationDir, $fileName);

    $urlBase = join('/', $pathElements);

    $fullUrl = $request->request->get('full_url', false);
    $url = (($fullUrl || env('UPLOADS_FULL_URL', false)) ? env('APP_URL').'/' : '' ).$urlBase.'/'.$fileName;

    $file = new $this->Model;
    $file->FileName = $url;
    $file->Description = $request->request->get('description', '');
    $file->Checksum = $cksum;
    $file->Store();

    return \App\Http\Response::Data($url, 201, JSON_UNESCAPED_SLASHES);
  }

  // /**
  //  * Gets all files uploaded
  //  *
  //  * @param  Request  $request
  //  * @return \Response
  //  */
  // public function GetAll(Request $request) {
  //   $path = env('UPLOADS_PATH', 'up');

  //   $files = array_values(array_diff(scandir('public/'.$path), ['.', '..']));
  //   foreach ($files as &$value) {
  //     $value = (env('UPLOADS_FULL_URL', false) ? env('APP_URL').'/' : '' ).$path.'/'.$value;
  //   }

  //   return \App\Http\Response::Data(['Array' => $files], 200, JSON_UNESCAPED_SLASHES);
  // }

  /**
   * Removes a file uploaded by its name or url
   *
   * @param  Request  $request
   * @return \Response
   */
  protected function DeleteFiles($array) {
    $fileNames = [];
    foreach ($array as $value) {
      $fileName = str_replace(env('APP_URL').'/', '', $value);
      $path = env('UPLOADS_PATH', 'up');

      if (!preg_match("/^{$path}\//", $fileName) || !file_exists($fileName)) {
        // pass
        // throw new \App\Exceptions\AppException(422, 'File does not exist');
      }
      $fileNames[] = $fileName;
    }

    foreach ($fileNames as $fileName) {
      unlink($fileName);
    }
  }

  public function DeleteById(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Id' => 'required|numeric',
    ]);

    $id = request('Id');
    $file = $this->Model::GetById($id);
    if (is_null($file)) {
      return \App\Http\Response::Error("File with id {$id} was not found");
    }

    $current_user = auth()->user();
    if ($current_user->Role < 1000 && $current_user->Id != $file->UserId) {
      throw new \App\Exceptions\AppException(422, 'Not allowed to delete.');
    }

    $this->DeleteFiles([$file->FileName]);
    $this->Model::DeleteArray([$id]);
    return \App\Http\Response::Data(null);
  }

  public function DeleteArray(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Ids' => 'required|array',
    ]);

    $ids = request('Ids');
    $files = $this->Model::GetArray($ids);

    $current_user = auth()->user();

    $files->each(function($file) use ($current_user) {
      if ($current_user->Role < 1000 && $current_user->Id != $file->UserId) {
        $id = $file->Id;
        throw new \App\Exceptions\AppException(422, "Not allowed to delete file with Id {$id}. Aborting full batch.");
      }
    });

    $fileNames = $files->pluck('FileName');

    $this->DeleteFiles($fileNames->toArray());
    $this->Model::DeleteArray($ids);

    return \App\Http\Response::Data(null);
  }

  /**
   * Function: sanitize
   * Returns a sanitized string, typically for URLs.
   *
   * Parameters:
   *     $string - The string to sanitize.
   *     $force_lowercase - Force the string to lowercase?
   *     $anal - If set to *true*, will remove all non-alphanumeric characters.
   */
  public function sanitize($string, $force_lowercase = false, $anal = false) {
      $strip = array("~", "`", "!", "@", "#", "$", "%", "^", "&", "*", "(", ")", "=", "+", "[", "{", "]",
                     "}", "\\", "|", ";", ":", "\"", "'", "&#8216;", "&#8217;", "&#8220;", "&#8221;", "&#8211;", "&#8212;",
                     "â€”", "â€“", ",", "<", ">", "/", "?", "..");
      $clean = trim(str_replace($strip, "", strip_tags($string)));
      $clean = preg_replace('/\s+/', "-", $clean);
      $clean = ($anal) ? preg_replace("/[^a-zA-Z0-9]/", "", $clean) : $clean ;
      return ($force_lowercase) ?
          (function_exists('mb_strtolower')) ?
              mb_strtolower($clean, 'UTF-8') :
              strtolower($clean) :
          $clean;
  }

  /**
   * ----------------------
   * TGenBase functionality
   * ----------------------
   */
  public function GetByFileName(Request $request) {
    $fail = \App\Http\Request::ValidateInput($request, [
      'FileName' => 'required',

    ]);
    if ($fail) return $fail;

    return \App\Http\Response::Data($this->Model::GetByFileName(request('FileName')));
  }

  public function GetByDescription(Request $request) {
    $fail = \App\Http\Request::ValidateInput($request, [
      'Description' => 'required',

    ]);
    if ($fail) return $fail;

    return \App\Http\Response::Data($this->Model::GetByDescription(request('Description')));
  }

  public function GetByChecksum(Request $request) {
    $fail = \App\Http\Request::ValidateInput($request, [
      'Checksum' => 'required',

    ]);
    if ($fail) return $fail;

    return \App\Http\Response::Data($this->Model::GetByChecksum(request('Checksum')));
  }

  public function GetByCreatedAt(Request $request) {
    $fail = \App\Http\Request::ValidateInput($request, [
      'CreatedAt' => 'required',

    ]);
    if ($fail) return $fail;

    return \App\Http\Response::Data($this->Model::GetByCreatedAt(request('CreatedAt')));
  }

  public function GetByUpdatedAt(Request $request) {
    $fail = \App\Http\Request::ValidateInput($request, [
      'UpdatedAt' => 'required',

    ]);
    if ($fail) return $fail;

    return \App\Http\Response::Data($this->Model::GetByUpdatedAt(request('UpdatedAt')));
  }

  public function GetByUserId(Request $request) {
    $fail = \App\Http\Request::ValidateInput($request, [
      'UserId' => 'required',

    ]);
    if ($fail) return $fail;

    return \App\Http\Response::Data($this->Model::GetByUserId(request('UserId')));
  }
}

