<?php

namespace App\Providers;

use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Contracts\Auth\UserProvider;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Hash;

use App\AuthToken;
use App\User;

class TokenUserProvider implements UserProvider
{
  private $token;
  private $user;

  public function __construct (User $user = null, AuthToken $token = null) {
    $this->user = $user ? $user : new User;
    $this->token = $token ? $token : new AuthToken;
  }

  public function retrieveById ($identifier) {
    return $this->user->GetById($identifier);
  }

  public function retrieveByToken ($identifier, $token) {
    $token = $this->token->GetWithIdentifier($identifier, $token);
    if (!$token)
      return null;

    if ($token->expires_at < now()) {
      $token->DeleteAndForget();
      return null;
    }

    return $this->user->GetById($token->user_id);
  }

  public function updateRememberToken (Authenticatable $user, $token) {
    // update via remember token not necessary
  }

  public function retrieveByCredentials (array $credentials) {
    $user = $this->user->GetByEmail($credentials['Email'])->first();
    if (!$user)
      return null;

    if (Hash::check($credentials['Password'], $user->Password)) {
      return $user;
    }

    return null;
  }

  public function validateCredentials (Authenticatable $user, array $credentials) {
    $plain = $credentials['Password'];

    return Hash::check($plain, $user->getAuthPassword());
  }
}