<?php

namespace App;

use Illuminate\Notifications\Notifiable;
use Illuminate\Auth\Authenticatable;
use Illuminate\Auth\MustVerifyEmail;
use Illuminate\Auth\Passwords\CanResetPassword;
use Illuminate\Foundation\Auth\Access\Authorizable;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\CanResetPassword as CanResetPasswordContract;

use App\BaseModel;
use App\BaseModelVersioned;

/**
 * User model derived from BaseModel to support versioned logic
 */
class User extends BaseModelVersioned implements AuthenticatableContract,
    AuthorizableContract,
    CanResetPasswordContract
{
    use Notifiable;
    use Authenticatable, Authorizable, CanResetPassword, MustVerifyEmail;

    /**
     * Column name for CreatedAt attribute
     *
     * @return strung
     */
    public function getCreatedAtColumn() {
        return 'CreatedAt';
    }

    /**
     * Column name for UpdatedAt attribute
     *
     * @return strung
     */
    public function getUpdatedAtColumn() {
        return 'UpdatedAt';
    }

    /**
     * Table name for this model
     *
     * @var strung
     */
    protected $table = 'User';

    /**
     * Default column for dictionary building
     *
     * @var string
     */
    public const dictionaryColumn = 'FullName';

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'Id', 'FullName', 'Email', 'Password', 'Address', 'Role', 'Status', 'CreatedAt', 'UpdatedAt'
    ];

    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = [
        'SeqNo', 'Password',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
    ];

    /**
     * Has many relation to \App\AuthToken model
     *
     * @return Illuminate\Database\Eloquent\Relations\HasMany
     */
    public function AuthTokens()
    {
        return $this->hasMany('App\AuthToken', 'user_id', 'Id');
    }

  /**
   * Get versioned entity by FileName with version Id
   *
   * @param  mixed  FileName
   * @param  int|null  $rid
   * @return \App\File|mixed
   */
  public static function GetByFullName($FullName) {
    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetByFullName', [$FullName,$perPage,$page], 600, function () use ($FullName, $perPage) {

      if (is_null($perPage))
        return static::GetBy()->where('FullName', 'like', '%'.$FullName.'%')->get();
      else
        return static::GetBy()->where('FullName', 'like', '%'.$FullName.'%')->paginate($perPage);

    });
  }

  /**
   * Get versioned entity by FileName with version Id
   *
   * @param  mixed  FileName
   * @param  int|null  $rid
   * @return \App\File|mixed
   */
  public static function GetByAddress($Address) {
    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetByAddress', [$Address,$perPage,$page], 600, function () use ($Address, $perPage) {

      if (is_null($perPage))
        return static::GetBy()->where('Address', 'like', '%'.$Address.'%')->get();
      else
        return static::GetBy()->where('Address', 'like', '%'.$Address.'%')->paginate($perPage);

    });
  }

  /**
   * Get versioned entity by FileName with version Id
   *
   * @param  mixed  FileName
   * @param  int|null  $rid
   * @return \App\File|mixed
   */
  public static function GetByEmail($Email) {
    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetByEmail', [$Email,$perPage,$page], 600, function () use ($Email, $perPage) {

      if (is_null($perPage))
        return static::GetBy()->where('Email', 'like', '%'.$Email.'%')->get();
      else
        return static::GetBy()->where('Email', 'like', '%'.$Email.'%')->paginate($perPage);

    });
  }

  /**
   * Get versioned entity by FileName with version Id
   *
   * @param  mixed  FileName
   * @param  int|null  $rid
   * @return \App\File|mixed
   */
  public static function GetByRole($Role) {
    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetByRole', [$Role,$perPage,$page], 600, function () use ($Role, $perPage) {

      if (is_null($perPage))
        return static::GetBy()->where('Role', $Role)->get();
      else
        return static::GetBy()->where('Role', $Role)->paginate($perPage);

    });
  }

  /**
   * Get versioned entity by FileName with version Id
   *
   * @param  mixed  FileName
   * @param  int|null  $rid
   * @return \App\File|mixed
   */
  public static function GetByStatus($Status) {
    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetByStatus', [$Status,$perPage,$page], 600, function () use ($Status, $perPage) {

      if (is_null($perPage))
        return static::GetBy()->where('Status', $Status)->get();
      else
        return static::GetBy()->where('Status', $Status)->paginate($perPage);

    });
  }

  /**
   * Get versioned entity by CreatedAt with version Id
   *
   * @param  mixed  CreatedAt
   * @param  int|null  $rid
   * @return \App\Module|mixed
   */
  public static function GetByCreatedAt($CreatedAt) {
    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetByCreatedAt', [$CreatedAt,$perPage,$page], 600, function () use ($CreatedAt, $perPage) {

      if (is_null($perPage))
        return static::GetBy()->where('CreatedAt', $CreatedAt)->get();
      else
        return static::GetBy()->where('CreatedAt', $CreatedAt)->paginate($perPage);

    });
  }

  /**
   * Get versioned entity by UpdatedAt with version Id
   *
   * @param  mixed  UpdatedAt
   * @param  int|null  $rid
   * @return \App\Module|mixed
   */
  public static function GetByUpdatedAt($UpdatedAt) {
    $perPage = request('per_page'); $page = request('page');
    return Cache::remember(static::class, 'GetByUpdatedAt', [$UpdatedAt,$perPage,$page], 600, function () use ($UpdatedAt, $perPage) {

      if (is_null($perPage))
        return static::GetBy()->where('UpdatedAt', $UpdatedAt)->get();
      else
        return static::GetBy()->where('UpdatedAt', $UpdatedAt)->paginate($perPage);

    });
  }
}
