import { mapGetters, mapActions } from 'vuex';
import * as UserRole from '@/components/Auth/UserRole.js'

const Shared = {
  name: 'shared',
  props: ['columnsTemplate', 'entityName', 'Rid'],

  data() {
    return {
      columns: this.columnsTemplate,
    }
  },

  created() {
  },

  mounted() {
  },

  filters: {
    truncate(value, length) {
      if (!value)
        return value;
      let stringValue = typeof value === 'string' ? value : JSON.stringify(value);
      return stringValue.length > length
        ? stringValue.substr(0, length) + '...'
        : stringValue
    }
  },

  computed: {
    ...mapGetters([
      'user', 'dictionaries', 'plainDictionaries'
    ]),

    canEdit() {
      return this.user.Role >= UserRole.Guest;
    },

    canDelete() {
      let cls = window.project.classes.find(val => val.name === this.entityName);
      return cls ? !cls.versioned && this.user.Role >= UserRole.Operator : false;
    },

    URL() {
      return (window[this.entityName + 'URL'] ||
              window['DefaultTGenBaseURL'] ||
              window.location.origin) + '/';
    },
  },

  methods: {
    ...mapActions(['getDictionary']),

    isRelation(column) {
      return column.relation;
    },

    isHasOne(column) {
      return this.isRelation(column) && (column.relationType === 'Has One' || column.relationType === undefined);
    },

    isHasMany(column) {
      return this.isRelation(column) && column.relationType === 'Has Many';
    },

    isManyToMany(column) {
      return this.isRelation(column) && column.relationType === 'Belongs to Many';
    },

    dictionaryAvailable(relation) {
      return this.plainDictionaries[relation] !== undefined && Object.keys(this.plainDictionaries[relation]).length !== 0;
    },

    filteredDictionary(relation, filter) {
      if (!this.dictionaryAvailable(relation)) return [];
      if (!filter || !filter.length) return this.dictionaries[relation];

      return this.dictionaries[relation].filter(val => val.Name.toLowerCase().includes(filter.toLowerCase()) ||
          val.Id.toString().includes(filter.toLowerCase()));
    },

    dictionaryColumn(entityName) {
      return App.schema.classes.find(val => val.name === entityName).dictionaryColumn || 'Name';
    },

    handleError(errorMessage) {
      if (errorMessage.response) {
        errorMessage = errorMessage.response.data.error;
      }

      console.log(errorMessage);
      App.showError(errorMessage);
    },

    passesRegex(value, regex) {
      if (value === undefined)
        return true;
      if (!regex || !regex.length)
        return true;

      value = String(value);
      let result = value.match(regex);
      if (result && result.length) {
        return true;
      }
      else {
        return false;
      }
    },

    regexDanger(value, regex) {
      return this.passesRegex(value, regex) ? '' : 'is-danger';
    },

    dangerMessage(value, regex) {
      return this.passesRegex(value, regex) ? '' : `Value does not pass validation rule ${regex}`;
    },

    checkFileType(uri, allowedTypes) {
      if (!uri || !uri.length) return false;
      return allowedTypes.some(val => uri.toLowerCase().endsWith(val));
    },

    isImage(uri) {
      return this.checkFileType(uri, ['.jpg', '.jpeg', '.png']);
    },

    isRoot(uri) {
      return this.checkFileType(uri, ['.root']);
    },

    isPdf(uri) {
      return this.checkFileType(uri, ['.pdf']);
    },

    isSpreadSheet(uri) {
      return this.checkFileType(uri, ['.csv', '.xls', '.xlsx']);
    },

    fileSource(raw) {
      if (!raw || raw.indexOf('://') > 0)
        return raw;
      else
        return this.URL + raw;
    },

    arxivSource(raw) {
      raw = String(raw);
      if (!raw) return raw;
      if (raw.indexOf('://arxiv.org/abs') > 0) {
        return raw.replace('abs', 'pdf') + '.pdf';
      }

      return 'https://arxiv.org/pdf/'+raw+'.pdf' ;
    },
  },
}

export default Shared