import axios from 'axios';
export default function Interceptors () {
  axios.interceptors.request.use((config) => {
    config.headers = config.headers || {};
    config.headers['Schema-Version'] = window.project.version;
    config.headers['Client-Version'] = window.project.generatorVersion;
    config.headers['Client-Type'] = 'Web';
    config.headers['X-Requested-With'] = 'XMLHttpRequest';

    let token = window.localStorage.getItem('auth-token')
    if (token) {
        config.headers['auth-token'] = token;
    }

    return config;
  });

  axios.interceptors.response.use((response) => {
    let serverVersion = response.headers['schema-version'];
    if (serverVersion && (serverVersion !== window.project.version))
      console.log(`Schema version mismatch: Server has ${serverVersion}`
                 +` Client has ${window.project.version}`);

    return response;
  }, (error) => {
    if (error.response.status === 401) {
      window.localStorage.removeItem('auth-token');
      window.sessionStorage.removeItem('auth-user');
    }

    return Promise.reject(error);
  });
}