export default {
  download(filename, element) {
    let text = document.getElementById(element).innerText;
    this.downloadData(filename, text);
  },

  downloadData(filename, data)
  {
    let blob = new Blob([data], {type: 'text/csv'});
    if (window.navigator.msSaveOrOpenBlob) {
      window.navigator.msSaveBlob(blob, filename);
    } else {
      let elem = window.document.createElement('a');
      elem.href = window.URL.createObjectURL(blob);
      elem.download = filename;
      document.body.appendChild(elem);
      elem.click();
      document.body.removeChild(elem);
    }
  },

  downloadFromWebService(path, params, timeout, onSuccess, onError) {
    let baseUrl = window.labviewWSURL;
    let req = new XMLHttpRequest();
    req.open('POST', baseUrl + path, true);
    req.timeout = timeout;
    req.responseType = "arraybuffer";
    req.onload = (dlEvent) => {
      if (dlEvent.target.status == 200)
      {
        if (onSuccess) {
          onSuccess(dlEvent.target.response);
        }
      } else {
        if (onError) {
          onError(String.fromCharCode.apply(null, new Uint8Array(dlEvent.target.response)));
        }
      }
    };
    req.onerror = req.ontimeout = (dlEvent) => {
      if (onError) {
        onError('Can\'t connect to LabVIEW web service at ' + baseUrl + '. Is it running?');
      }
    };

    req.send(JSON.stringify(params, null, 2));
  },
}
