const Parser = {
  parseText(text, classDef, staticData) {
    if (text.match(/ClassDef(.*);/)) {
      this.parseHeader(text, classDef, staticData);
    } else if (text.match(/ClassImp(.*);/)) {
      this.parseCxx(text, classDef, staticData);
    }
  },

  parseHeader(text, classDef, staticData) {
    [, classDef.name] = text.match(/class ([a-zA-Z0-9_]+) : public TGenBaseGenericParSet/) || [,''];
    [, classDef.title] = text.match(/const char\* title = "([a-zA-Z0-9_\s]+)",/) || [,''];
    [, classDef.context] = text.match(/\s{10}const char\* context = "(.*)",/) || [,''];
    [, classDef.detectorType] = text.match(/TGenBaseDetector::Detector_t detid = TGenBaseDetector::([a-zA-Z0-9_]+),/) || [,''];
    [, classDef.dataType] = text.match(/DataType::DataType_t dataid = DataType::([a-zA-Z0-9_]+),/) || [,''];
    [, classDef.version] = text.match(/ClassDef\(.*,.*(\d+)\);/) || [,''];

    let regexp; let match;

    let foundIncludes = [];
    regexp = /#include "([a-zA-Z0-9_\/]+).h"/g;
    match = regexp.exec(text);
    while (match != null) {
      foundIncludes.push(match[1]);
      match = regexp.exec(text);
    }
    classDef.includes = foundIncludes.filter( (cls) => !staticData.defaultIncludes.includes(cls) );

    let foundRelations = [];
    let foundRelationClasses = [];
    let foundRelationTypes = [];
    regexp = /\s+[a-zA-Z_\d]+[\*]?\s+f([a-zA-Z0-9_]+);[\s+]?\/\/\! transient relation to ([a-zA-Z0-9_]+), (Has (One|Many))/g;
    match = regexp.exec(text);
    while (match != null) {
      foundRelations.push(match[1]);
      foundRelationClasses.push(match[2]);
      foundRelationTypes.push(match[3]);
      match = regexp.exec(text);
    }
    let ignoredFields = foundRelations.concat(foundRelations.map(val => val + 'Id'));
    classDef.relations = foundRelations.map( (val, index) => 
      ( {className: foundRelationClasses[index], type: foundRelationTypes[index], property: foundRelations[index]} ) );

    let foundTypes = [];
    let foundProperties = [];
    let foundDims = [];
    regexp = /\s{4}(?!return)([a-zA-Z_\d\*]+)\s+(?!fId)f([a-zA-Z0-9_]+);(?!.*relation.*)([\s+]?\/\/\[f([a-zA-Z0-9_]+)\])?/g;
    match = regexp.exec(text);
    while (match != null) {
      let foundField = match[2] || '';
      if (ignoredFields.includes(foundField) || ignoredFields.includes(foundField+'Id')) {
      } else {
        foundTypes.push(match[1]);
        foundProperties.push(match[2]);
        foundDims.push(match[4]);
      }
      match = regexp.exec(text);
    }

    let properties = foundTypes.map((val, index) => new Property( { name: foundProperties[index], cxxType: foundTypes[index], defaultValue: '', dbType: '', dim: foundDims[index] || '', jsonType: ''} ) );
    properties.forEach(function(val, index) {
      let def;
      if (val.isPointer()) {
        def = [ {cxxType:val.cxxType, dbType:'TEXT', defaultValue:'NULL', jsonType: ''} ];
      }
      else {
        def = staticData.types.filter(type => type.cxxType == val.cxxType);
      }

      if (def && def.length) {
        val.defaultValue = def[0].defaultValue;
        val.dbType = def[0].dbType;
        val.jsonType = def[0].jsonType;
      }
    });

    classDef.properties = properties;
  },

  parseCxx(text, classDef, staticData) {
    let foundDefaults = [];
    let foundProperties = [];
    let foundDbFields = [];
    let foundDbTypes = [];

    let regexp; let match;

    regexp = /\s+f([a-zA-Z]+)\((.*)\)[,]?/g;
    match = regexp.exec(text);
    while (match != null) {
      foundProperties.push(match[1]);
      foundDefaults.push(match[2]);
      match = regexp.exec(text);
    }

    regexp = /\s+(?!SEQNO|ROW_ID|INT|NOT)([A-Z_]+)\s+([A-Z_\s]+)[NOT NULL]?,";/g;
    match = regexp.exec(text);
    while (match != null) {
      foundDbFields.push(match[1]);
      foundDbTypes.push(match[2]);
      match = regexp.exec(text);
    }

    for (let fIndex = 0; fIndex < foundProperties.length; fIndex++)
    {
      for (let sIndex = 0; sIndex < classDef.properties.length; sIndex ++) {
        if (foundProperties[fIndex] == classDef.properties[sIndex].name) {
          classDef.properties[sIndex].defaultValue = foundDefaults[fIndex];
        }
      }
    }

    for (let fIndex = 0; fIndex < foundDbFields.length; fIndex++)
    {
      for (let sIndex = 0; sIndex < classDef.properties.length; sIndex ++) {
        if (this.toCamelCase(foundDbFields[fIndex]) == classDef.properties[sIndex].name) {
          classDef.properties[sIndex].dbType = foundDbTypes[fIndex];
        }
      }
    }
  },

  capitalize(string) {
    // rowId -> RowId
    return string.charAt(0).toUpperCase() + string.slice(1);
  },

  toCamelCase(string) {
    // from table case; ROW_ID -> RowId
    return string.toLowerCase().replace(/_([a-z])/g, function (g) { return g[1].toUpperCase(); }).capitalize();
  },
}

export default Parser
