import ClassDef from '@/js/ClassDef.js'

class Project {
  constructor(object) {
    if (!object)
      object = {};
    this.generatorVersion = "1.4.0";
    this.version = object.version || "1.0.0";
    this.name = object.name || '';
    this.url = object.url || '';
    this.classes = object.classes ? object.classes.map(val => new ClassDef(val)) : [];
  };

  copyFrom(other) {
    this.generatorVersion = "1.4.0";
    this.version = other.version;
    this.name = other.name;
    this.url = other.url;
    this.classes = other.classes;
  };

  store() {
    localStorage.setItem('Project', JSON.stringify(this));
  };

  static restore() {
    let value = localStorage.getItem('Project');
    if (value) {
      let obj = JSON.parse(value);
      return new Project(obj);
    }

    return new Project( {} );
  };

  clear() {
    this.generatorVersion = "1.4.0";
    this.version = "1.0.0"
    this.name = '';
    this.url = '';
    this.classes = [];
  };

  good() {
    if (!this.name.length) return false;
    if (!this.classes || !this.classes.length) return false;
    if (this.classes.some(val => !val.good()) ) return false;

    return true;
  };

  incrementVersion() {
    let ver = this.version.split('.');
    ver[2] = Number(ver[2]) + 1;
    this.version = ver.join(".");
  };

  addClass(index) {
    this.classes.splice(index, 0, new ClassDef( {} ));
  };

  copyClass(index) {
    let newClass = new ClassDef(this.classes[index]);
    newClass.name += "_Copy";
    this.classes.splice(index + 1, 0, newClass);
  };

  removeClass(index) {
    this.classes.splice(index, 1);
  };

  allClassNames() {
    return this.classes.map(val => val.name);
  };

  getClassDefByName(name) {
    return this.classes.find(val => val.name == name);
  };

  fileName() {
    return this.name + '.json';
  };

  linkdefFileName() {
    return this.name + 'LinkDef.h';
  };

  restProjectName() {
    return this.name + 'REST';
  };

  restHeaderFileName() {
    return this.restProjectName() + '.h';
  };

  restCxxFileName() {
    return this.restProjectName() + '.cxx';
  };

  restHeaderGuard() {
    return this.restProjectName().toUpperCase() + '_H';
  };

  restServerAppName() {
    return this.restProjectName() + '.wt';
  };

  classLibFileName() {
    return this.name + '.LabVIEW.zip';
  };

  cmakeProjectName() {
    return this.name.toUpperCase();
  };

  manyToManyRelations() {
    let hash = {};

    let relations = [];
    this.classes.forEach(cls => {
      cls.relations.filter(val => val.isManyToManyRelation()).forEach(relation => {
        let names = [relation.className, cls.name].sort((a, b) => a.localeCompare(b));
        let combo = `${names[0]}_${names[1]}`;
        if (!hash[combo]) {
          relations.push([names[0], names[1]]);
          hash[combo] = combo;
        }
      });
    });

    relations = relations.sort((a, b) => (a[0]+a[1]).localeCompare(b[0]+b[1]));

    return relations;
  };
}

export default Project
