import Project from '@/js/Project.js'
import Property from '@/js/Property.js'
import Relation from '@/js/Relation.js'
import Static from '@/js/Static.js'

class Schema {
  constructor(object) {
    if (!object)
      object = {};
    this.name = object.name || '';
    this.project = object.project || {};
    this.builtIn = object.builtIn || new Project(require('@/js/BuiltIn.json'));
    this.externals = object.externals || [];
    this.generatorVersion = "1.3.0";
    this.version = object.version || "1.0.0";
    this.url = object.url || '';
  };

  copyFrom(other) {
    this.name = other.name || '';
    this.project = other.project || {};
    this.builtIn = other.builtIn || {};
    this.externals = other.externals || [];
    this.generatorVersion = "1.3.0";
    this.version = other.version || "1.0.0";
    this.url = other.url || '';
  };

  store() {
    localStorage.setItem('Schema', JSON.stringify(this));
  };

  static restore() {
    let value = localStorage.getItem('Schema');
    if (value) {
      let obj = JSON.parse(value);
      return new Schema(obj);
    }

    return new Schema( {} );
  };

  clear() {
    this.generatorVersion = "1.2.0";
    this.version = "1.0.0"
    this.name = '';
    this.classes = [];
    this.versioned = true;
  };

  good() {
    if (!this.name.length) return false;
    if (!this.project.good()) return false;

    return true;
  };

}

export default Schema
