import * as AccessLevel from '@/components/Auth/AccessLevel.js'

import File from '@/components/File/File.vue'
import FileNew from '@/components/File/New.vue'
import FileGetById from '@/components/File/GetById.vue'
import FileGetAll from '@/components/File/GetAll.vue'
import FileGetAllVersions from '@/components/File/GetAllVersions.vue'
import FileGetWithQuery from '@/components/File/GetWithQuery.vue'

import FileGetByFileName from '@/components/File/GetByFileName.vue'
import FileGetByDescription from '@/components/File/GetByDescription.vue'
import FileGetByChecksum from '@/components/File/GetByChecksum.vue'
import FileGetByCreatedAt from '@/components/File/GetByCreatedAt.vue'
import FileGetByUpdatedAt from '@/components/File/GetByUpdatedAt.vue'
import FileGetByUserId from '@/components/File/GetByUserId.vue'

export default function FileRoutes (router) {
  router.addRoutes([
    {
      path: '/File/',
      name: 'file',
      component: File,
      meta: {
        access: AccessLevel.Operator,
      },
    },
    {
      path: '/File/New/',
      name: 'file-new',
      component: FileNew,
      meta: {
        access: AccessLevel.Operator,
      },
      props: (route) => ({ Rid: Number(route.query.Rid) || (Date.now() / 1000|0) })
    },
    {
      path: '/File/GetById/:Id',
      name: 'file-getbyid',
      component: FileGetById,
      meta: {
        access: AccessLevel.Operator,
      },
      props: (route) => ({ Rid: Number(route.query.Rid) || (Date.now() / 1000|0), Id: Number(route.params.Id) || 0 })
    },
    {
      path: '/File/GetAllVersions/:Id',
      name: 'file-getallversions',
      component: FileGetAllVersions,
      meta: {
        access: AccessLevel.Operator,
      },
      props: (route) => ({ Id: Number(route.params.Id) || 0 })
    },
    {
      path: '/File/GetAll',
      name: 'file-getall',
      component: FileGetAll,
      meta: {
        access: AccessLevel.Operator,
      },
      props: (route) => ({ Rid: Number(route.query.Rid) || (Date.now() / 1000|0) })
    },
    {
      path: '/File/GetWithQuery/',
      name: 'file-getwithquery',
      component: FileGetWithQuery,
      meta: {
        access: AccessLevel.Operator,
      },
      props: (route) => ({ Rid: Number(route.query.Rid) || (Date.now() / 1000|0), Query: route.params.Query })
    },

    {
      path: '/File/GetByFileName/:FileName',
      name: 'file-getbyfilename',
      component: FileGetByFileName,
      meta: {
        access: AccessLevel.Operator,
      },
      props: (route) => ({ Rid: Number(route.query.Rid) || (Date.now() / 1000|0), FileName: route.params.FileName })
    },

    {
      path: '/File/GetByDescription/:Description',
      name: 'file-getbydescription',
      component: FileGetByDescription,
      meta: {
        access: AccessLevel.Operator,
      },
      props: (route) => ({ Rid: Number(route.query.Rid) || (Date.now() / 1000|0), Description: route.params.Description })
    },

    {
      path: '/File/GetByChecksum/:Checksum',
      name: 'file-getbychecksum',
      component: FileGetByChecksum,
      meta: {
        access: AccessLevel.Operator,
      },
      props: (route) => ({ Rid: Number(route.query.Rid) || (Date.now() / 1000|0), Checksum: route.params.Checksum })
    },

    {
      path: '/File/GetByCreatedAt/:CreatedAt',
      name: 'file-getbycreatedat',
      component: FileGetByCreatedAt,
      meta: {
        access: AccessLevel.Operator,
      },
      props: (route) => ({ Rid: Number(route.query.Rid) || (Date.now() / 1000|0), CreatedAt: route.params.CreatedAt })
    },

    {
      path: '/File/GetByUpdatedAt/:UpdatedAt',
      name: 'file-getbyupdatedat',
      component: FileGetByUpdatedAt,
      meta: {
        access: AccessLevel.Operator,
      },
      props: (route) => ({ Rid: Number(route.query.Rid) || (Date.now() / 1000|0), UpdatedAt: route.params.UpdatedAt })
    },

    {
      path: '/File/GetByUserId/:UserId',
      name: 'file-getbyuserid',
      component: FileGetByUserId,
      meta: {
        access: AccessLevel.Operator,
      },
      props: (route) => ({ Rid: Number(route.query.Rid) || (Date.now() / 1000|0), UserId: route.params.UserId })
    },

  ])
}

