import axios from 'axios';

import Vue from 'vue';
import Vuex from 'vuex';
import Snackbar from 'buefy/src/components/snackbar';
Vue.use(Vuex);

const store = new Vuex.Store({
  state: {
    user: undefined,//{ FullName: '',  Email: '', Address: '', Status: 0, Role: 0 },
    token: undefined,//'',
    loggedIn: false,
    dictionaries: {},
    plainDictionaries: {},
  },

  getters: {
    user: state => {
      return state.user;
    },

    token: state => {
      return state.token;
    },

    loggedIn: state => {
      return state.loggedIn;
    },

    dictionaries: state => {
      return state.dictionaries;
    },

    plainDictionaries: state => {
      return state.plainDictionaries;
    },
  },

  actions: {
    logout (context) {
      context.commit('logout');
    },

    restoreState (context) {
      // workaround for the fact that we load the store prior to App
      window.App = { store: this };
      let user = JSON.parse(window.localStorage.getItem('auth-user')) || undefined;
      let token = window.localStorage.getItem('auth-token') || undefined;

      context.commit('setUser', user);
      context.commit('setToken', token);

      if (token)
      {
        // refresh user info
        axios.post((window.DefaultTGenBaseURL || window.location.origin) + '/User/Get', {})
          .then(function (response) {
            if (!response.data.error) {
              context.commit('loginWithUser', { user: response.data.data, token: token });
            } else {
              App.showError("Session expired, you need to relogin");
              context.commit('logout');
            }
          }).catch(function (error) {
            if (error.response && (error.response.status === 401 || error.response.status === 403)) {
              App.showError("Session expired, you need to relogin");
              context.commit('logout');
            }
          });
      }
    },

    async getDictionary( { commit, state }, { relation, URL = null }) {
      if (!URL) {
        URL = (window.DefaultTGenBaseURL || window.location.origin) + '/';
      }

      if (state.dictionaries[relation])
        return;

      axios.post(URL + relation + '/GetDictionary')
        .then(function (response) {

          let dict = response.data.data;
          let keys = Object.keys(dict);
          let values = Object.values(dict);
          let options = keys.map((val, idx) => { return {Id: Number(keys[idx]), Name: values[idx].toString()} } );
          commit('setDictionary', { relation, dict: options, plainDict: dict });
        })
        .catch(function (error) {
          console.log(error);
        });
    },
  },

  mutations: {
    setUser (state, user) {
      state.user = user;
    },

    setToken (state, token) {
      state.token = token;
    },

    loginWithUser (state, payload) {
      state.user = payload.user;
      state.token = payload.token;
      state.loggedIn = true;

      window.localStorage.setItem('auth-user', JSON.stringify(state.user));
      window.localStorage.setItem('auth-token', payload.token);
    },

    logout (state) {
      state.user = undefined;//{ FullName: '',  Email: '', Address: '', Status: 0, Role: 0 };
      state.token = undefined; //'';
      state.loggedIn = false;

      window.localStorage.removeItem('auth-user');
      window.localStorage.removeItem('auth-token');
    },

    setDictionary (state, { relation, dict, plainDict }) {
      state.plainDictionaries = {...state.plainDictionaries, [relation]: plainDict};
      state.dictionaries = {...state.dictionaries, [relation]: dict};
    },
  }
});

export default store;