#/bin/bash

pushd "$(dirname ${BASH_SOURCE[0]})"

PORT=$1
[[ -z $PORT ]] && PORT=8008

# Generate docroot out of vue.js project
if [[ -d gui/vue ]]; then

  pushd gui/vue
    CKSUM=`ls -lAR src | $MD5`
    if [[ ! -f .md5 ]] || [[ "$CKSUM" != `cat .md5` ]]; then
      npm install
      npm run build
    fi
    echo "$CKSUM" > .md5
  popd

  rsync -a -P gui/docroot/ server/public/
fi

if [[ ! -d .git ]]; then
  git init > /dev/null
  git config user.name tgenbase
  git config user.email tgenbase@gsi.de
  git add . && git commit -m "Init" > /dev/null
fi

pushd server

  composer install --no-plugins --no-scripts

  if [[ ! -f .env ]]; then
    SED="sed -i"
    if [[ "$OSTYPE" == "darwin"* ]]; then
      SED="sed -i .back"
    fi

    cp .env.example .env

    DBNAME=`pwd -P`/database/database.sqlite
    touch $DBNAME

    $SED "s|DB_CONNECTION=mysql|DB_CONNECTION=sqlite|" .env
    $SED "s|DB_DATABASE=homestead|DB_DATABASE=$DBNAME|" .env
    php artisan key:generate

    php artisan migrate
    php artisan db:seed
  fi

  php artisan migrate
  php artisan serve --host=0.0.0.0 --port=$PORT

popd

popd