<?php

namespace App;

/**
 * Wrapper over cache stores which both support and do not support tags
 */
class Cache
{
  static $_supportsTags = null;

  public static function supportsTags() {
    if (static::$_supportsTags === null) {
      if (\Illuminate\Support\Facades\Cache::store()->getStore() instanceOf \Illuminate\Cache\FileStore ||
          \Illuminate\Support\Facades\Cache::store()->getStore() instanceOf \Illuminate\Cache\DatabaseStore)
      {
        static::$_supportsTags = false;
      } else {
        static::$_supportsTags = true;
      }
    }

    return static::$_supportsTags;
  }

  public static function get($class, $method, $cacheKey) {
    if (!env('USE_CACHE', true))
      return null;

    if (!is_string($cacheKey)) $cacheKey = json_encode($cacheKey);
    if (request('With',''))
      $cacheKey = $cacheKey.json_encode(request('With'));
    if (static::supportsTags()) {
      return \Illuminate\Support\Facades\Cache::tags([$class, $method])->get(join('_', [$method, $cacheKey]));
    } else {
      return \Illuminate\Support\Facades\Cache::get(join('_', [$class, $method, $cacheKey]));
    }
  }

  public static function remember($class, $method, $cacheKey, $seconds, \Closure $defaultValue) {
    if (!env('USE_CACHE', true))
      return $defaultValue();

    if (!is_string($cacheKey)) $cacheKey = json_encode($cacheKey);
    if (request('With',''))
      $cacheKey = $cacheKey.json_encode(request('With'));
    if (static::supportsTags()) {
      return \Illuminate\Support\Facades\Cache::tags([$class, $method])->remember(join('_', [$method, $cacheKey]), $seconds, $defaultValue);
    } else {
      return \Illuminate\Support\Facades\Cache::remember(join('_', [$class, $method, $cacheKey]), $seconds, $defaultValue);
    }
  }

  public static function forget($class, $method, $cacheKey) {
    if (!env('USE_CACHE', true))
      return;

    if (!is_string($cacheKey)) $cacheKey = json_encode($cacheKey);
    if (request('With',''))
      $cacheKey = $cacheKey.json_encode(request('With'));
    if (static::supportsTags()) {
      return \Illuminate\Support\Facades\Cache::tags([$class, $method])->forget(join('_', [$method, $cacheKey]));
    } else {
      return \Illuminate\Support\Facades\Cache::forget(join('_', [$class, $method, $cacheKey]));
    }
  }

  public static function flush($class) {
    if (!env('USE_CACHE', true))
      return;

    if (static::supportsTags()) {
      return \Illuminate\Support\Facades\Cache::tags([$class])->flush();
    } else {
      return \Illuminate\Support\Facades\Cache::flush();
    }
  }
}
