<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Validator;
use App\Http\Response;

class BaseController extends Controller
{
  protected $Model = \App\BaseModel::class;

  public function GetById(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Id' => 'required|integer',
    ]);

    return \App\Http\Response::Data($this->Model::GetById(request('Id')));
  }

  public function GetAll(Request $request) {
    \App\Http\Request::ValidateInput($request, []);

    return \App\Http\Response::Data($this->Model::GetAll());
  }

  public function GetAllVersions(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Id' => 'required|integer',
    ]);

    return \App\Http\Response::Data($this->Model::GetAllVersions(request('Id')));
  }

  public function GetArray(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Ids' => 'required|array',
      'Ids.*'  => 'required|integer',
    ]);

    return \App\Http\Response::Data($this->Model::GetArray(request('Ids')));
  }

  public function GetWithQuery(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Query' => 'required|array',
    ]);

    return \App\Http\Response::Data($this->Model::GetwithQuery(request('Query')));
  }

  public function GetDictionary(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Column' => 'nullable|string',
    ]);

    return \App\Http\Response::Data($this->Model::GetDictionary(request('Column')), 200, JSON_FORCE_OBJECT);
  }

  public function Store(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Object' => 'required',
    ]);

    $Object = new $this->Model((array)request('Object'));

    return \App\Http\Response::Data($Object->Store());
  }

  public function StoreArray(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Array' => 'required|array',
    ]);

    $Array = collect(request('Array'));

    return \App\Http\Response::Data($this->Model::StoreArray($Array));
  }

  public function DeleteById(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Id' => 'required|integer',
    ]);

    return \App\Http\Response::Data($this->Model::DeleteById(request('Id')));
  }

  public function DeleteArray(Request $request) {
    \App\Http\Request::ValidateInput($request, [
      'Ids' => 'required|array',
      'Ids.*'  => 'required|integer',
    ]);

    return \App\Http\Response::Data($this->Model::DeleteArray(request('Ids')));
  }
}
