<?php

namespace App\Http\Middleware;

use Closure;

/**
* Middleware to check versions between server and client
* Schema version is checked for exact match if CHECK_SCHEMA_VERSION environment variable is set to true
*
* @param \Illuminate\Http\Request $request
* @param \Closure $next
*
* @return mixed
*/
class CheckVersionsMiddleware
{
    /**
     * Handle an incoming request.
     *
     * @param \Illuminate\Http\Request $request
     * @param \Closure $next
     *
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        if (env('CHECK_SCHEMA_VERSION', false)) {
            $clientVersion = $request->headers->get('Schema-Version', '1.0.0');
            if ($clientVersion !== \App\Version::schema) {
                throw new \App\Exceptions\AppException(422, 'Schema version mismatch: '.
                    'Server has '.\App\Version::schema.' Client has '.$clientVersion);
            }
        }

        $response = $next($request);
        $response->header('Server-Version', \App\Version::server);
        $response->header('Schema-Version', \App\Version::schema);

        return $response;
    }
}
